/*
 * Copyright 2015 The gRPC Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package io.emqx.exhook;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import io.emqx.exhook.MongoDB.MongoUtils;
import io.grpc.Server;
import io.grpc.ServerBuilder;
import io.grpc.stub.StreamObserver;
import com.google.protobuf.ByteString;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;
import java.util.concurrent.TimeUnit;
import java.util.logging.Logger;

import com.mongodb.client.MongoClient;
import com.mongodb.client.MongoDatabase;
import com.mongodb.client.MongoCollection;
import com.mongodb.client.model.Filters;
import org.bson.Document;

public class ExServer {
    private static final Logger logger = Logger.getLogger(ExServer.class.getName());
    private static SimpleDateFormat simpleDateFormat =new SimpleDateFormat("yyyy-MM-dd HH:mm:ss SSS" );
    private Server server;

    private void start() throws IOException {
        /* The port on which the server should run */
        int port = 9000;

        server = ServerBuilder.forPort(port)
                .addService(new HookProviderImpl())
                .build()
                .start();
        logger.info("Server started, listening on " + port);
        Runtime.getRuntime().addShutdownHook(new Thread() {
            @Override
            public void run() {
                // Use stderr here since the logger may have been reset by its JVM shutdown hook.
                System.err.println("*** shutting down gRPC server since JVM is shutting down");
                try {
                    ExServer.this.stop();
                } catch (InterruptedException e) {
                    e.printStackTrace(System.err);
                }
                System.err.println("*** server shut down");
            }
        });
    }

    private void stop() throws InterruptedException {
        if (server != null) {
            server.shutdown().awaitTermination(30, TimeUnit.SECONDS);
        }
    }

    /**
     * Await termination on the main thread since the grpc library uses daemon threads.
     */
    private void blockUntilShutdown() throws InterruptedException {
        if (server != null) {
            server.awaitTermination();
        }
    }

    /**
     * Main launches the server from the command line.
    */
    public static void main(String[] args) throws IOException, InterruptedException {
        final ExServer server = new ExServer();
        server.start();
        server.blockUntilShutdown();
    }

    static class HookProviderImpl extends HookProviderGrpc.HookProviderImplBase {

        public void DEBUG(String fn, Object req) {
            System.out.printf(fn + ", request: " + req);
        }

        @Override
        public void onProviderLoaded(ProviderLoadedRequest request, StreamObserver<LoadedResponse> responseObserver) {
            DEBUG("onProviderLoaded", request);
            HookSpec[] specs = {
                    HookSpec.newBuilder().setName("client.connect").build(),
                    HookSpec.newBuilder().setName("client.connack").build(),
                    HookSpec.newBuilder().setName("client.connected").build(),
                    HookSpec.newBuilder().setName("client.disconnected").build(),
                    HookSpec.newBuilder().setName("client.authenticate").build(),
                    HookSpec.newBuilder().setName("client.authorize").build(),
                    HookSpec.newBuilder().setName("client.subscribe").build(),
                    HookSpec.newBuilder().setName("client.unsubscribe").build(),

                    HookSpec.newBuilder().setName("session.created").build(),
                    HookSpec.newBuilder().setName("session.subscribed").build(),
                    HookSpec.newBuilder().setName("session.unsubscribed").build(),
                    HookSpec.newBuilder().setName("session.resumed").build(),
                    HookSpec.newBuilder().setName("session.discarded").build(),
                    HookSpec.newBuilder().setName("session.takenover").build(),
                    HookSpec.newBuilder().setName("session.terminated").build(),

                    HookSpec.newBuilder().setName("message.publish").build(),
                    HookSpec.newBuilder().setName("message.delivered").build(),
                    HookSpec.newBuilder().setName("message.acked").build(),
                    HookSpec.newBuilder().setName("message.dropped").build()
            };
            LoadedResponse reply = LoadedResponse.newBuilder().addAllHooks(Arrays.asList(specs)).build();
            responseObserver.onNext(reply);
            responseObserver.onCompleted();
        }

        @Override
        public void onProviderUnloaded(ProviderUnloadedRequest request, StreamObserver<EmptySuccess> responseObserver) {
            DEBUG("onProviderUnloaded", request);
            EmptySuccess reply = EmptySuccess.newBuilder().build();
            responseObserver.onNext(reply);
            responseObserver.onCompleted();
        }

        @Override
        public void onClientConnect(ClientConnectRequest request, StreamObserver<EmptySuccess> responseObserver) {
            DEBUG("onClientConnect", request);
            Thread MessageLogs = new Thread() {
                public void run() {
                    try {
                        JSONObject parseObject = new JSONObject();
                        parseObject.put("node",request.getConninfo().getNode());
                        parseObject.put("clientid",request.getConninfo().getClientid());
                        parseObject.put("peerhost",request.getConninfo().getClientid());
                        parseObject.put("sockport",request.getConninfo().getSockport());
                        parseObject.put("proto_name",request.getConninfo().getProtoName());
                        parseObject.put("proto_ver",request.getConninfo().getProtoVer());
                        parseObject.put("keepalive",request.getConninfo().getKeepalive());
                        parseObject.put("timestamp",simpleDateFormat.format(new Date()));
                        MongoUtils.save(parseObject,"MQTT_ClientConnect");
                    } catch (Exception e) {
                        e.printStackTrace();
                    }

                }
            };
            MessageLogs.start();

            EmptySuccess reply = EmptySuccess.newBuilder().build();
            responseObserver.onNext(reply);
            responseObserver.onCompleted();
        }

        @Override
        public void onClientConnack(ClientConnackRequest request, StreamObserver<EmptySuccess> responseObserver) {
            DEBUG("onClientConnack", request);
            Thread MessageLogs = new Thread() {
                public void run() {
                    try {
                        JSONObject parseObject = new JSONObject();
                        parseObject.put("node",request.getConninfo().getNode());
                        parseObject.put("clientid",request.getConninfo().getClientid());
                        parseObject.put("peerhost",request.getConninfo().getClientid());
                        parseObject.put("sockport",request.getConninfo().getSockport());
                        parseObject.put("proto_name",request.getConninfo().getProtoName());
                        parseObject.put("proto_ver",request.getConninfo().getProtoVer());
                        parseObject.put("username",request.getConninfo().getUsername());
                        parseObject.put("keepalive",request.getConninfo().getKeepalive());
                        parseObject.put("timestamp",simpleDateFormat.format(new Date()));
                        MongoUtils.save(parseObject,"MQTT_ClientConnack");
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
            };
            MessageLogs.start();

            EmptySuccess reply = EmptySuccess.newBuilder().build();
            responseObserver.onNext(reply);
            responseObserver.onCompleted();
        }

        @Override
        public void onClientConnected(ClientConnectedRequest request, StreamObserver<EmptySuccess> responseObserver) {
            DEBUG("onClientConnected", request);
            EmptySuccess reply = EmptySuccess.newBuilder().build();
            responseObserver.onNext(reply);
            responseObserver.onCompleted();
        }

        @Override
        public void onClientDisconnected(ClientDisconnectedRequest request, StreamObserver<EmptySuccess> responseObserver) {
            DEBUG("onClientDisconnected", request);
            Thread MessageLogs = new Thread() {
                public void run() {
                    try {
                        JSONObject parseObject = new JSONObject();
                        parseObject.put("node",request.getClientinfo().getNode());
                        parseObject.put("clientid",request.getClientinfo().getClientid());
                        parseObject.put("peerhost",request.getClientinfo().getClientid());
                        parseObject.put("sockport",request.getClientinfo().getSockport());
                        parseObject.put("anonymous",request.getClientinfo().getAnonymous());
                        parseObject.put("protocol",request.getClientinfo().getProtocol());
                        parseObject.put("username",request.getClientinfo().getUsername());
                        parseObject.put("reason",request.getReason());
                        parseObject.put("timestamp",simpleDateFormat.format(new Date()));
                        MongoUtils.save(parseObject,"MQTT_ClientDisconnected");

                        //更新神的连接状态
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
            };
            MessageLogs.start();

            EmptySuccess reply = EmptySuccess.newBuilder().build();
            responseObserver.onNext(reply);
            responseObserver.onCompleted();
        }

        @Override
        public void onClientAuthenticate(ClientAuthenticateRequest request, StreamObserver<ValuedResponse> responseObserver) {
            DEBUG("onClientAuthenticate", request);
            Thread MessageLogs = new Thread() {
                public void run() {
                    try {
                        JSONObject parseObject = new JSONObject();
                        parseObject.put("node",request.getClientinfo().getNode());
                        parseObject.put("clientid",request.getClientinfo().getClientid());
                        parseObject.put("peerhost",request.getClientinfo().getClientid());
                        parseObject.put("sockport",request.getClientinfo().getSockport());
                        parseObject.put("protocol",request.getClientinfo().getProtocol());
                        parseObject.put("anonymous",request.getClientinfo().getAnonymous());
                        parseObject.put("username",request.getClientinfo().getUsername());
                        parseObject.put("password",request.getClientinfo().getPassword());
                        parseObject.put("timestamp",simpleDateFormat.format(new Date()));
                        MongoUtils.save(parseObject,"MQTT_ClientAuthenticate");
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
            };
            MessageLogs.start();

            ValuedResponse reply = ValuedResponse.newBuilder()
                                                 .setBoolResult(true)
                                                 .setType(ValuedResponse.ResponsedType.STOP_AND_RETURN)
                                                 .build();
            responseObserver.onNext(reply);
            responseObserver.onCompleted();
        }

        @Override
        public void onClientAuthorize(ClientAuthorizeRequest request, StreamObserver<ValuedResponse> responseObserver) {
            DEBUG("onClientAuthorize", request);
            ValuedResponse reply = ValuedResponse.newBuilder()
                                                 .setBoolResult(true)
                                                 .setType(ValuedResponse.ResponsedType.STOP_AND_RETURN)
                                                 .build();

            responseObserver.onNext(reply);
            responseObserver.onCompleted();
        }

        @Override
        public void onClientSubscribe(ClientSubscribeRequest request, StreamObserver<EmptySuccess> responseObserver) {
            DEBUG("onClientSubscribe", request);
            EmptySuccess reply = EmptySuccess.newBuilder().build();
            responseObserver.onNext(reply);
            responseObserver.onCompleted();
        }

        @Override
        public void onClientUnsubscribe(ClientUnsubscribeRequest request, StreamObserver<EmptySuccess> responseObserver) {
            DEBUG("onClientUnsubscribe", request);
            EmptySuccess reply = EmptySuccess.newBuilder().build();
            responseObserver.onNext(reply);
            responseObserver.onCompleted();
        }

        @Override
        public void onSessionCreated(SessionCreatedRequest request, StreamObserver<EmptySuccess> responseObserver) {
            DEBUG("onSessionCreated", request);
            EmptySuccess reply = EmptySuccess.newBuilder().build();
            responseObserver.onNext(reply);
            responseObserver.onCompleted();
        }

        @Override
        public void onSessionSubscribed(SessionSubscribedRequest request, StreamObserver<EmptySuccess> responseObserver) {
            DEBUG("onSessionSubscribed", request);
            EmptySuccess reply = EmptySuccess.newBuilder().build();
            responseObserver.onNext(reply);
            responseObserver.onCompleted();
        }

        @Override
        public void onSessionUnsubscribed(SessionUnsubscribedRequest request, StreamObserver<EmptySuccess> responseObserver) {
            DEBUG("onSessionUnsubscribed", request);
            EmptySuccess reply = EmptySuccess.newBuilder().build();
            responseObserver.onNext(reply);
            responseObserver.onCompleted();
        }

        @Override
        public void onSessionResumed(SessionResumedRequest request, StreamObserver<EmptySuccess> responseObserver) {
            DEBUG("onSessionResumed", request);
            EmptySuccess reply = EmptySuccess.newBuilder().build();
            responseObserver.onNext(reply);
            responseObserver.onCompleted();
        }

        @Override
        public void onSessionDiscarded(SessionDiscardedRequest request, StreamObserver<EmptySuccess> responseObserver) {
            DEBUG("onSessionDdiscarded", request);
            EmptySuccess reply = EmptySuccess.newBuilder().build();
            responseObserver.onNext(reply);
            responseObserver.onCompleted();
        }

        @Override
        public void onSessionTakenover(SessionTakenoverRequest request, StreamObserver<EmptySuccess> responseObserver) {
            DEBUG("onSessionTakenover", request);
            EmptySuccess reply = EmptySuccess.newBuilder().build();
            responseObserver.onNext(reply);
            responseObserver.onCompleted();
        }

        @Override
        public void onSessionTerminated(SessionTerminatedRequest request, StreamObserver<EmptySuccess> responseObserver) {
            DEBUG("onSessionTerminated", request);
            EmptySuccess reply = EmptySuccess.newBuilder().build();
            responseObserver.onNext(reply);
            responseObserver.onCompleted();
        }

        @Override
        public void onMessagePublish(MessagePublishRequest request, StreamObserver<ValuedResponse> responseObserver) {
            DEBUG("onMessagePublish", request);
            MongoUtils.save(request);
            Thread MessageLogs = new Thread() {
                public void run() {
                    try {
                        JSONObject parseObject = new JSONObject();
                        parseObject.put("node",request.getMessage().getNode());
                        parseObject.put("message_id",request.getMessage().getId());
                        parseObject.put("from",request.getMessage().getFrom());
                        parseObject.put("payload",request.getMessage().getPayload().toString("UTF-8"));
                        parseObject.put("topic",request.getMessage().getTopic());
                        parseObject.put("headers",request.getMessage().getHeadersMap());
                        parseObject.put("meta",request.getMeta().toString());
                        parseObject.put("timestamp",simpleDateFormat.format(new Date(request.getMessage().getTimestamp())));
                        MongoUtils.save(parseObject,"MQTT_MessagePublish");
                    } catch (Exception e) {
                        e.printStackTrace();
                    }

                }
            };
            MessageLogs.start();
            /* toDoList
             * 1、解密消息体
             * 2、判断消息是否是合法消息
             * 3、判断消息是否重复
             * 4、判断是否是告警
             * 5、发送到内部MQ
             *
             */

            ByteString bstr = ByteString.copyFromUtf8("hardcode payload by exhook-svr-java :)");

            Message nmsg = Message.newBuilder()
                                  .setId     (request.getMessage().getId())
                                  .setNode   (request.getMessage().getNode())
                                  .setFrom   (request.getMessage().getFrom())
                                  .setTopic  (request.getMessage().getTopic())
                                  .setPayload(bstr).build();


            ValuedResponse reply = ValuedResponse.newBuilder()
                                                 .setType(ValuedResponse.ResponsedType.STOP_AND_RETURN)
                                                 .setMessage(nmsg).build();
            responseObserver.onNext(reply);
            responseObserver.onCompleted();
        }

// case2: stop publish the 't/d' messages
//        @Override
//        public void onMessagePublish(MessagePublishRequest request, StreamObserver<ValuedResponse> responseObserver) {
//            DEBUG("onMessagePublish", request);
//
//            Message nmsg = request.getMessage();
//            if ("t/d".equals(nmsg.getTopic())) {
//                ByteString bstr = ByteString.copyFromUtf8("");
//                nmsg = Message.newBuilder()
//                              .setId     (request.getMessage().getId())
//                              .setNode   (request.getMessage().getNode())
//                              .setFrom   (request.getMessage().getFrom())
//                              .setTopic  (request.getMessage().getTopic())
//                              .setPayload(bstr)
//                              .putHeaders("allow_publish", "false").build();
//            }
//
//            ValuedResponse reply = ValuedResponse.newBuilder()
//                                                 .setType(ValuedResponse.ResponsedType.STOP_AND_RETURN)
//                                                 .setMessage(nmsg).build();
//            responseObserver.onNext(reply);
//            responseObserver.onCompleted();
//        }

        @Override
        public void onMessageDelivered(MessageDeliveredRequest request, StreamObserver<EmptySuccess> responseObserver) {
            DEBUG("onMessageDelivered", request);
            EmptySuccess reply = EmptySuccess.newBuilder().build();
            responseObserver.onNext(reply);
            responseObserver.onCompleted();
        }

        @Override
        public void onMessageAcked(MessageAckedRequest request, StreamObserver<EmptySuccess> responseObserver) {
            DEBUG("onMessageAcked", request);
            EmptySuccess reply = EmptySuccess.newBuilder().build();
            responseObserver.onNext(reply);
            responseObserver.onCompleted();
        }

        @Override
        public void onMessageDropped(MessageDroppedRequest request, StreamObserver<EmptySuccess> responseObserver) {
            DEBUG("onMessageDropped", request);
            EmptySuccess reply = EmptySuccess.newBuilder().build();
            responseObserver.onNext(reply);
            responseObserver.onCompleted();
        }
    }
}
