package io.emqx.exhook.MongoDB.paging;

import java.util.List;

/**
 * 分页对象
 *
 * @Author Sugar
 * @Version 2017年3月30日 下午8:08:58
 */
public class Page implements IPage {
    private static final long serialVersionUID = 1L;

    private long totalCount;// 总记录数
    private int pageCount;// 总页数
    private int pageSize;// 每页记录数
    private int pageNo;// 当前页页码，从1开始
    private int prevPageNo;// 上一页页码，为0时无上一页
    private int nextPageNo;// 下一页页码，为0时无下一页
    private List<?> list;// 本页数据集
    private String orderBy;
    private Class clazz;
    private long maxSize;//最大显示记录数
    private int endPageNo;//最后一页页码

    public Page() {
        pageNo = 1;
        pageSize = 20;
    }

    public Page(int pageSize, int pageNo) {
        this.pageSize = pageSize;
        this.pageNo = (pageNo < 1 ? 1 : pageNo);
    }


    /**
     * @param totalCount
     *            数据总记录数
     * @param pageSize
     *            每页显示多少条数据
     * @param pageNo
     *            当前第几页，从1开始
     * @param list 当前数据列表
     */
    public <T> Page(long totalCount, int pageSize, int pageNo, List<T> list) {
        this.totalCount = totalCount;
        this.pageSize = pageSize;
        this.pageNo = (pageNo < 1 ? 1 : pageNo);
        this.list = list;
        init();
    }

    /**
     *
     * @param pageSize 每页显示多少条数据
     * @param pageNo 当前第几页，从1开始
     * @param dataList dataList 所有数据列表
     */
    public <T> Page(int pageSize, int pageNo, List<T> dataList) {
        if (dataList == null) {
            return;
        }
        this.totalCount = dataList.size();
        this.pageSize = pageSize;
        this.pageNo = (pageNo < 1 ? 1 : pageNo);
        int start = pageSize * (this.pageNo - 1);
        int end = start + pageSize;
        if (end <= totalCount) {
            this.list = dataList.subList(start, end);
        } else if (start < totalCount) {
            this.list = dataList.subList(start, (int) totalCount);
        }
        init();
    }

    public long getTotalCount() {
        return totalCount;
    }

    public void setTotalCount(long totalCount) {
        this.totalCount = totalCount;
        init();
    }

    public int getPageCount() {
        return pageCount;
    }

    public int getPageSize() {
        return pageSize;
    }

    public void setPageSize(int pageSize) {
        if (this.pageSize != pageSize) {
            this.pageSize = pageSize;
            init();
        }
    }

    public int getPageNo() {
        return pageNo;
    }

    public void setPageNo(int pageNo) {
        this.pageNo = pageNo;
    }

    /**
     * @return the orderBy
     */
    public String getOrderBy() {
        return orderBy;
    }

    /**
     * @param orderBy the orderBy to set
     */
    public void setOrderBy(String orderBy) {
        this.orderBy = orderBy;
    }

    @SuppressWarnings("unchecked")
    public <T> List<T> getList() {
        return (List<T>) list;
    }

    public <T> void setList(List<T> list) {
        this.list = list;
    }

    public int getPrevPageNo() {
        return prevPageNo;
    }

    public void setPrevPageNo(int prevPageNo) {
        this.prevPageNo = prevPageNo;
    }

    public int getNextPageNo() {
        return nextPageNo;
    }

    public void setNextPageNo(int nextPageNo) {
        this.nextPageNo = nextPageNo;
    }

    public void init() {
        if (totalCount % this.pageSize == 0) {
            this.pageCount = (int) (totalCount / this.pageSize);
        } else {
            this.pageCount = (int) (totalCount / this.pageSize + 1);
        }
        this.endPageNo = this.pageCount;
        if (this.maxSize > 0 && this.maxSize < this.totalCount) {//如果限制了最大记录数，最大页数只能以最大记录数计算
            this.endPageNo = (int)(this.maxSize / this.pageSize);
        }
        if (this.pageNo < 1) {
            this.pageNo = 1;
        } else if (this.endPageNo > 0 && this.pageNo > this.endPageNo) {
            this.pageNo = this.endPageNo;
        }
        this.prevPageNo = this.pageNo - 1;
        if (this.pageNo == this.endPageNo) {
            this.nextPageNo = 0;
        } else {
            this.nextPageNo = this.pageNo + 1;
        }
    }

    public <T> Class<T> getClazz() {
        return clazz;
    }

    public void setClazz(Class clazz) {
        this.clazz = clazz;
    }

    public void setMaxSize(long maxSize) {
        this.maxSize = maxSize;
    }

    public void reset() {
        this.pageNo = 1;
        this.prevPageNo = 0;
        this.nextPageNo = 0;
        this.pageCount = 0;
        this.list = null;
        this.totalCount = 0;
    }
}